import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { IonicModule } from '@ionic/angular';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

interface Artwork {
  id: string;
  image: string;
  title: string;
  description: string;
  avatarUrl: string;
  userName: string;
  date: string;
  likes: number;
  category: string;
  comments?: Comment[];
}

interface Comment {
  id: string;
  userId: string;
  userName: string;
  avatarUrl: string;
  content: string;
  date: string;
  likes: number;
}

@Component({
  selector: 'app-art-detail',
  templateUrl: './art-detail.component.html',
  styleUrls: ['./art-detail.component.scss'],
  standalone: true,
  imports: [IonicModule, CommonModule, FormsModule]
})
export class ArtDetailComponent implements OnInit {
  artwork?: Artwork;
  isLiked = false;
  isSaved = false;
  commentText = '';

  constructor(
    private route: ActivatedRoute,
    private router: Router
  ) {
    const navigation = this.router.getCurrentNavigation();
    if (navigation?.extras.state) {
      this.artwork = navigation.extras.state['artwork'] as Artwork;
      // 添加示例评论
      this.artwork.comments = [
        {
          id: '1',
          userId: 'user1',
          userName: '艺术评论家',
          avatarUrl: 'assets/img/lunbo2.png',
          content: '这幅作品的构图和用色都非常出色，展现了艺术家深厚的功底。',
          date: '2024-01-15',
          likes: 23
        },
        {
          id: '2',
          userId: 'user2',
          userName: '美术爱好者',
          avatarUrl: 'assets/img/lunbo3.png',
          content: '光影处理很到位，整体氛围感很强。',
          date: '2024-01-14',
          likes: 15
        }
      ];
    }
  }

  ngOnInit() {
    if (!this.artwork) {
      const artId = this.route.snapshot.paramMap.get('id');
      console.log('Art ID:', artId);
    }
  }

  toggleLike() {
    this.isLiked = !this.isLiked;
    if (this.artwork) {
      this.artwork.likes += this.isLiked ? 1 : -1;
    }
  }

  toggleSave() {
    this.isSaved = !this.isSaved;
    this.showToast(this.isSaved ? '已收藏' : '已取消收藏');
  }

  async shareArtwork() {
    if (navigator.share) {
      try {
        await navigator.share({
          title: this.artwork?.title,
          text: this.artwork?.description,
          url: window.location.href
        });
      } catch (error) {
        this.showToast('分享失败，请重试');
      }
    } else {
      await navigator.clipboard.writeText(window.location.href);
      this.showToast('链接已复制到剪贴板');
    }
  }

  async submitComment() {
    if (!this.commentText.trim()) {
      this.showToast('请输入评论内容');
      return;
    }
    // 这里添加提交评论的逻辑
    this.showToast('评论已提交');
    this.commentText = '';
  }

  private async showToast(message: string) {
    const toast = document.createElement('ion-toast');
    toast.message = message;
    toast.duration = 2000;
    toast.position = 'top';
    document.body.appendChild(toast);
    await toast.present();
  }

  goBack() {
    this.router.navigate(['/tabs/tab3']);
  }
}
