import { Injectable } from '@angular/core';
import { CloudUser } from 'src/lib/ncloud';

@Injectable({
    providedIn: 'root',
})
export class UserService {
    private currentUser: CloudUser | null = null;

    constructor() {
        this.currentUser = new CloudUser(); // 初始化当前用户
    }

    // 获取当前用户
    getCurrentUser(): CloudUser | null {
        return this.currentUser;
    }

    // 设置当前用户
    setCurrentUser(user: CloudUser | null): void {
        this.currentUser = user;
    }

    // 登录方法
    async login(username: string, password: string): Promise<CloudUser | null> {
        if (!this.currentUser) {
            // 如果 currentUser 未定义，创建一个新的 CloudUser 实例
            this.currentUser = new CloudUser();
        }

        const user = await this.currentUser.login(username, password); // 调用登录方法
        this.setCurrentUser(user);
        return user;
    }

    // 登出方法
    async logout(): Promise<void> {
        await this.currentUser?.logout();
        this.setCurrentUser(null);
    }
}